@@ -24,6 +24,8 @@ from dagster import (
 )
 from dagster import _check as check
 from dagster import get_dagster_logger, op
+from dagster.core.asset_defs.load_assets_from_modules import prefix_assets
+from dagster.core.definitions.events import CoercibleToAssetKeyPrefix
 from dagster.core.definitions.metadata import RawMetadataValue
 from dagster.core.errors import DagsterInvalidSubsetError
 
@@ -62,11 +64,11 @@ def _select_unique_ids_from_manifest_json(
         import dbt.graph.selector as graph_selector
         from dbt.contracts.graph.manifest import Manifest
         from networkx import DiGraph
-    except ImportError:
-        check.failed(
+    except ImportError as e:
+        raise check.CheckError(
             "In order to use the `select` argument on load_assets_from_dbt_manifest, you must have"
             "`dbt-core >= 1.0.0` and `networkx` installed."
-        )
+        ) from e
 
     class _DictShim(dict):
         """Shim to enable hydrating a dictionary into a dot-accessible object"""
@@ -101,15 +103,34 @@ def _get_node_name(node_info: Mapping[str, Any]):
     return "__".join([node_info["resource_type"], node_info["package_name"], node_info["name"]])
 
 
-def _get_node_asset_key(node_info):
-    if node_info.get("schema") is not None:
-        components = [node_info["schema"], node_info["name"]]
+def _get_node_asset_key(node_info: Mapping[str, Any]) -> AssetKey:
+    """By default:
+
+        dbt sources: a dbt source's key is the union of its source name and its table name
+        dbt models: a dbt model's key is the union of its model name and any schema configured on
+    the model itself.
+    """
+    if node_info["resource_type"] == "source":
+        components = [node_info["source_name"], node_info["name"]]
     else:
-        components = [node_info["name"]]
+        configured_schema = node_info["config"].get("schema")
+        if configured_schema is not None:
+            components = [configured_schema, node_info["name"]]
+        else:
+            components = [node_info["name"]]
 
     return AssetKey(components)
 
 
+def _get_node_group_name(node_info: Mapping[str, Any]) -> Optional[str]:
+    """A node's group name is subdirectory that it resides in"""
+    fqn = node_info.get("fqn", [])
+    # the first component is the package name, and the last component is the model name
+    if len(fqn) < 3:
+        return None
+    return fqn[1]
+
+
 def _get_node_description(node_info):
     code_block = textwrap.indent(node_info["raw_sql"], "    ")
     description_sections = [
@@ -132,6 +153,7 @@ def _dbt_nodes_to_assets(
 ) -> AssetsDefinition:
 
     outs: Dict[str, Out] = {}
+    group_names: Dict[AssetKey, str] = {}
     asset_ins: Dict[AssetKey, Tuple[str, In]] = {}
 
     asset_deps: Dict[AssetKey, Set[AssetKey]] = {}
@@ -165,13 +187,20 @@ def _dbt_nodes_to_assets(
         outs[node_name] = Out(
             description=_get_node_description(node_info),
             io_manager_key=io_manager_key,
-            metadata=_columns_to_metadata(node_info["columns"]),
+            metadata=_get_node_metadata(node_info),
             is_required=False,
         )
         out_name_to_node_info[node_name] = node_info
 
+        asset_key = node_info_to_asset_key(node_info)
+
         # set the asset dependencies for this asset
-        asset_deps[node_info_to_asset_key(node_info)] = cur_asset_deps
+        asset_deps[asset_key] = cur_asset_deps
+
+        # set the group for this asset
+        group_name = _get_node_group_name(node_info)
+        if group_name is not None:
+            group_names[asset_key] = group_name
 
     # prevent op name collisions between multiple dbt multi-assets
     op_name = f"run_dbt_{package_name}"
@@ -187,23 +216,26 @@ def _dbt_nodes_to_assets(
     )
     def dbt_op(context):
         dbt_output = None
-        try:
-            # in the case that we're running everything, opt for the cleaner selection string
-            if len(context.selected_output_names) == len(outs):
-                subselect = select
-            else:
-                # for each output that we want to emit, translate to a dbt select string by converting
-                # the out to it's corresponding fqn
-                subselect = [
-                    ".".join(out_name_to_node_info[out_name]["fqn"])
-                    for out_name in context.selected_output_names
-                ]
 
+        # clean up any run results from the last run
+        context.resources.dbt.remove_run_results_json()
+
+        # in the case that we're running everything, opt for the cleaner selection string
+        if len(context.selected_output_names) == len(outs):
+            subselect = select
+        else:
+            # for each output that we want to emit, translate to a dbt select string by converting
+            # the out to it's corresponding fqn
+            subselect = [
+                ".".join(out_name_to_node_info[out_name]["fqn"])
+                for out_name in context.selected_output_names
+            ]
+
+        try:
             if use_build_command:
                 dbt_output = context.resources.dbt.build(select=subselect)
             else:
                 dbt_output = context.resources.dbt.run(select=subselect)
-
         finally:
             # in the case that the project only partially runs successfully, still attempt to generate
             # events for the parts that were successful
@@ -238,38 +270,37 @@ def _dbt_nodes_to_assets(
                     yield event
 
     return AssetsDefinition(
-        asset_keys_by_input_name={
+        keys_by_input_name={
             input_name: asset_key for asset_key, (input_name, _) in asset_ins.items()
         },
-        asset_keys_by_output_name={
+        keys_by_output_name={
             output_name: node_info_to_asset_key(out_name_to_node_info[output_name])
             for output_name in outs.keys()
         },
         node_def=dbt_op,
         can_subset=True,
         asset_deps=asset_deps,
+        group_names_by_key=group_names,
     )
 
 
-def _columns_to_metadata(columns: Mapping[str, Any]) -> Optional[Mapping[str, Any]]:
-    return (
-        {
-            "schema": MetadataValue.table_schema(
-                TableSchema(
-                    columns=[
-                        TableColumn(
-                            name=name,
-                            type=metadata.get("data_type") or "?",
-                            description=metadata.get("description"),
-                        )
-                        for name, metadata in columns.items()
-                    ]
-                )
+def _get_node_metadata(node_info: Mapping[str, Any]) -> Mapping[str, Any]:
+    metadata: Dict[str, Any] = {}
+    columns = node_info.get("columns", [])
+    if len(columns) > 0:
+        metadata["table_schema"] = MetadataValue.table_schema(
+            TableSchema(
+                columns=[
+                    TableColumn(
+                        name=column_name,
+                        type=column_info.get("data_type") or "?",
+                        description=column_info.get("description"),
+                    )
+                    for column_name, column_info in columns.items()
+                ]
             )
-        }
-        if len(columns) > 0
-        else None
-    )
+        )
+    return metadata
 
 
 def load_assets_from_dbt_project(
@@ -277,6 +308,8 @@ def load_assets_from_dbt_project(
     profiles_dir: Optional[str] = None,
     target_dir: Optional[str] = None,
     select: Optional[str] = None,
+    key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+    source_key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
     runtime_metadata_fn: Optional[
         Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, Any]]
     ] = None,
@@ -296,8 +329,12 @@ def load_assets_from_dbt_project(
             Defaults to a directory called "config" inside the project_dir.
         target_dir (Optional[str]): The target directory where DBT will place compiled artifacts.
             Defaults to "target" underneath the project_dir.
-        select (str): A DBT selection string for the models in a project that you want to include.
-            Defaults to "*".
+        select (Optional[str]): A DBT selection string for the models in a project that you want
+            to include. Defaults to "*".
+        key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all models in the dbt
+            project. Does not apply to sources.
+        source_key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all sources in the
+            dbt project. Does not apply to models.
         runtime_metadata_fn: (Optional[Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, Any]]]):
             A function that will be run after any of the assets are materialized and returns
             metadata entries for the asset, to be displayed in the asset catalog for that run.
@@ -311,35 +348,36 @@ def load_assets_from_dbt_project(
             for this asset, rather than `dbt run`.
 
     """
-    check.str_param(project_dir, "project_dir")
+    project_dir = check.str_param(project_dir, "project_dir")
     profiles_dir = check.opt_str_param(
         profiles_dir, "profiles_dir", os.path.join(project_dir, "config")
     )
     target_dir = check.opt_str_param(target_dir, "target_dir", os.path.join(project_dir, "target"))
+    select = check.opt_str_param(select, "select", "*")
 
     manifest_json, cli_output = _load_manifest_for_project(
-        project_dir, profiles_dir, target_dir, select or "*"
+        project_dir, profiles_dir, target_dir, select
     )
     selected_unique_ids: Set[str] = set(
         filter(None, (line.get("unique_id") for line in cli_output.logs))
     )
-
-    dbt_nodes = {**manifest_json["nodes"], **manifest_json["sources"]}
-    return [
-        _dbt_nodes_to_assets(
-            dbt_nodes,
-            select=select or "*",
-            selected_unique_ids=selected_unique_ids,
-            runtime_metadata_fn=runtime_metadata_fn,
-            io_manager_key=io_manager_key,
-            node_info_to_asset_key=node_info_to_asset_key,
-            use_build_command=use_build_command,
-        ),
-    ]
+    return load_assets_from_dbt_manifest(
+        manifest_json=manifest_json,
+        key_prefix=key_prefix,
+        source_key_prefix=source_key_prefix,
+        runtime_metadata_fn=runtime_metadata_fn,
+        io_manager_key=io_manager_key,
+        selected_unique_ids=selected_unique_ids,
+        select=select,
+        node_info_to_asset_key=node_info_to_asset_key,
+        use_build_command=use_build_command,
+    )
 
 
 def load_assets_from_dbt_manifest(
     manifest_json: Mapping[str, Any],
+    key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+    source_key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
     runtime_metadata_fn: Optional[
         Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, Any]]
     ] = None,
@@ -358,6 +396,10 @@ def load_assets_from_dbt_manifest(
     Args:
         manifest_json (Optional[Mapping[str, Any]]): The contents of a DBT manifest.json, which contains
             a set of models to load into assets.
+        key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all models in the dbt
+            project. Does not apply to sources.
+        source_key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all sources in the
+            dbt project. Does not apply to models.
         runtime_metadata_fn: (Optional[Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, Any]]]):
             A function that will be run after any of the assets are materialized and returns
             metadata entries for the asset, to be displayed in the asset catalog for that run.
@@ -388,14 +430,29 @@ def load_assets_from_dbt_manifest(
         # must resolve the selection string using the existing manifest.json data (hacky)
         selected_unique_ids = _select_unique_ids_from_manifest_json(manifest_json, select)
 
-    return [
-        _dbt_nodes_to_assets(
-            dbt_nodes,
-            runtime_metadata_fn=runtime_metadata_fn,
-            io_manager_key=io_manager_key,
-            select=select,
-            selected_unique_ids=selected_unique_ids,
-            node_info_to_asset_key=node_info_to_asset_key,
-            use_build_command=use_build_command,
-        )
-    ]
+    dbt_assets_def = _dbt_nodes_to_assets(
+        dbt_nodes,
+        runtime_metadata_fn=runtime_metadata_fn,
+        io_manager_key=io_manager_key,
+        select=select,
+        selected_unique_ids=selected_unique_ids,
+        node_info_to_asset_key=node_info_to_asset_key,
+        use_build_command=use_build_command,
+    )
+    if source_key_prefix:
+        if isinstance(source_key_prefix, str):
+            source_key_prefix = [source_key_prefix]
+        source_key_prefix = check.list_param(source_key_prefix, "source_key_prefix", of_type=str)
+        input_key_replacements = {
+            input_key: AssetKey(source_key_prefix + input_key.path)
+            for input_key in dbt_assets_def.keys_by_input_name.values()
+        }
+        dbt_assets = [
+            dbt_assets_def.with_prefix_or_group(input_asset_key_replacements=input_key_replacements)
+        ]
+    else:
+        dbt_assets = [dbt_assets_def]
+
+    if key_prefix:
+        dbt_assets = prefix_assets(dbt_assets, key_prefix)
+    return dbt_assets
